/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/

var SyncFilter = function()
{
	var sf = {};
	
	var allowedParentItems;
	var allowedChildItems;
	var filter;
	
	/**
		Initialize the page.
	*/
	sf.initPage = function()
	{
		var pageId = Doc.getPageId();		
		if (pageId == 'edit-page')
		{
			var frm = document.forms['syncfilter'];
			// Load info about possible parent and child items
			allowedParentItems = Data.json('page-data', 'allowed-parent-items');
			allowedChildItems = Data.json('page-data', 'allowed-child-items');
			if (allowedChildItems.length == 0) frm.sourceItemTransform[2] = null;
			if (allowedParentItems.length == 0) frm.sourceItemTransform[1] = null;
			
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', sf.save);
			Buttons.addClickHandler('close', App.closeWindow);
			
			Events.addEventHandler('sourceItemTransform', 'change', sf.sourceItemTransformOnChange);
			Events.addEventHandler('sourceItemType', 'change', sf.sourceItemTypeOnChange);
			
			Buttons.addClickHandler('btnModifyFilter', sf.modifyFilter);
			Events.addEventHandler('btnModifyFilter', 'base-selected-start', sf.sourceFilterModified);
			
			// Tab validation
			TabControl.addTabValidator('settings.info', sf.validateSyncFilter);
			sf.sourceItemTransformOnChange();
		}
	}


	sf.validateSyncFilter = function()
	{
		var frm = document.forms['syncfilter'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		if (!filter)
		{
			Forms.showNotification('btnModifyFilter', 'You must specify a filter');
			return false;
		}
		return true;
	}
	
	/**
		When the source transform change, we must update the
		source items selection list so that it only contains the
		parent or child items or the source item itself.
	 */
	sf.sourceItemTransformOnChange = function()
	{
		var frm = document.forms['syncfilter'];
		var transform = frm.sourceItemTransform.value;
		if (transform == 'NONE')
		{
			Doc.hide('parent-child-transform');
			Doc.show('none-transform', 'inline');
			frm.sourceItemType.length = 0;
			var listMemberType = Data.get('page-data', 'list-member-type');
			frm.sourceItemType[frm.sourceItemType.length] = new Option(listMemberType, listMemberType, true);
		}
		else
		{
			var allowedItems = transform == 'PARENT_TO_CHILD' ? allowedParentItems : allowedChildItems;
			frm.sourceItemType.length = 0;
			var sourceItemType = Data.get('page-data', 'source-item-type');
			for (var i = 0; i < allowedItems.length; i++)
			{
				var item = allowedItems[i];
				frm.sourceItemType[frm.sourceItemType.length] = new Option(item.title, item.name, item.name == sourceItemType);
			}
			Doc.show('parent-child-transform', 'inline');
			Doc.hide('none-transform');
		}
		sf.sourceItemTypeOnChange();
	}
	
	/**
		When the source item changes, we submit an AJAX query to get
		the current filter setting.
	*/
	sf.sourceItemTypeOnChange = function()
	{
		var frm = document.forms['syncfilter'];
		var url = 'ajax.jsp?ID='+App.getSessionId();
		url += '&cmd=GetCurrentFilter';
		url += '&sourceItemType='+frm.sourceItemType.value;
		url += '&sourceItemTransform='+frm.sourceItemTransform.value;
		
		var request = Ajax.getXmlHttpRequest();
		request.open("GET", url, true);
		Ajax.setReadyStateHandler(request, sf.filterInfoLoaded);
		request.send(null);
	}
	
	/**
		Display the current filter.
	*/
	sf.filterInfoLoaded = function(request)
	{
		var info = JSON.parse(request.responseText);
		if (info && info.status == 'ok')
		{
			filter = info.filter;
			Doc.element('filter-info').innerHTML = filter || '<i>no filter</i>';
		}
	}
	
	/**
		When the modify button is clicked we start by issuing an ajax
		request to copy the current filter to the current context.
		
		When the filter has been copied (sf.filterHasBeenCopied)
		the dialog is opened.
	*/
	sf.modifyFilter = function()
	{
		var frm = document.forms['syncfilter'];
		var url = 'ajax.jsp?ID='+App.getSessionId();
		url += '&cmd=CopySyncFilterToItemContext';
		url += '&sourceItemType='+frm.sourceItemType.value;

		var request = Ajax.getXmlHttpRequest();
		request.open("GET", url, true);
		Ajax.setReadyStateHandler(request, sf.filterHasBeenCopied);
		request.send(null);
	}
	
	// When we are sure the current filter is in the item context,
	// open the dialog
	sf.filterHasBeenCopied = function(request)
	{
		var frm = document.forms['syncfilter'];
		Dialogs.selectItem(frm.sourceItemType.value, 'btnModifyFilter', 'selectfilter');
	}
	
	/**
		When closing the dialog we react to the 'base-selected-start'
		event and issue an ajax request to copy the current context
		into the current filter.
	*/
	sf.sourceFilterModified = function(event)
	{
		var frm = document.forms['syncfilter'];
		var url = 'ajax.jsp?ID='+App.getSessionId();
		url += '&cmd=CopyItemContextToSyncFilter';
		url += '&sourceItemType='+frm.sourceItemType.value;
		url += '&sourceItemTransform='+frm.sourceItemTransform.value;

		var request = Ajax.getXmlHttpRequest();
		request.open("GET", url, true);
		Ajax.setReadyStateHandler(request, sf.filterInfoLoaded);
		request.send(null);
	}
	
	// Submit the form
	sf.save = function()
	{
		var frm = document.forms['syncfilter'];
		if (TabControl.validateActiveTab('settings'))
		{
			frm.submit();
		}
	}
		
	return sf;
}();

Doc.onLoad(SyncFilter.initPage);
